#include <iostream>
#include <fstream>
#include <cassert>
#include <ctime>
#include <sstream>
#include <string>
#include "IOException.hpp"
#include "Logger.hpp"


Kronos::Logger* Kronos::Logger::s_instance = nullptr;

void Kronos::Logger::Log(const std::string& msg) {
	Kronos::Logger::Log(Kronos::Logger::Level::INFO, msg);
}

void Kronos::Logger::Log(Kronos::Logger::Level level, const std::string& msg) {
	assert(Kronos::Logger::s_instance != nullptr);
	Kronos::Logger::s_instance->log(level, msg);
}

Kronos::Logger::Logger(Kronos::Logger::Level level, const char* filename, bool console)
		: m_level(level), m_console(console) {
	assert(Kronos::Logger::s_instance == nullptr);

	if(filename != nullptr) {
		m_logFile.open(filename, std::ofstream::app);
		if(!m_logFile.is_open()) {
			throw Kronos::Exception::IOException("Unable to open log file <" + std::string(filename) + ">");
		}
	}

	Kronos::Logger::s_instance = this;
}

Kronos::Logger::~Logger() {
	if(m_logFile.is_open()) {
		m_logFile.close();
	}
	Kronos::Logger::s_instance = nullptr;
}

void Kronos::Logger::setLevel(Kronos::Logger::Level level) {
	m_level = level;
}

void Kronos::Logger::log(Kronos::Logger::Level level, const std::string& msg) {
	if(level <= m_level) {
		time_t curTime = time(nullptr);
		std::stringstream ss;

		char dtStr[20];
		strftime(dtStr, 20, "%Y-%m-%d %H:%M:%S", localtime(&curTime));

		const char* lvlName = nullptr;
		if(level == Kronos::Logger::Level::ERROR) {
			lvlName = "ERROR";
		} else if(level == Kronos::Logger::Level::WARN) {
			lvlName = "WARN";
		} else if(level == Kronos::Logger::Level::DEBUG) {
			lvlName = "DEBUG";
		} else {
			lvlName = "INFO";
		}

		ss << "[" << dtStr << "]" << "[" << lvlName << "] : " << msg;

		if(m_logFile.is_open()) {
			m_logFile << ss.str() << std::endl;
		}

		if(m_console) {
			std::cout << ss.str() << std::endl;
		}
	}
}
