#include <vector>
#include <string>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include "Logger.hpp"
#include "XWindow.hpp"
#include "XDisplayInitException.hpp"
#include "XDisplayConnection.hpp"


Kronos::XDisplayConnection::XDisplayConnection(const char* name) {
	m_display = XOpenDisplay(name);
	if(m_display == nullptr) {
		throw Kronos::Exception::XDisplayInitException("Unable to open X display");
	}
}

Kronos::XDisplayConnection::~XDisplayConnection() {
	XCloseDisplay(m_display);
}

Display* Kronos::XDisplayConnection::getDisplay() {
	return m_display;
}

std::vector<Kronos::XWindow> Kronos::XDisplayConnection::getWindows() {
	std::vector<XWindow> windows;
	Atom propWin = XInternAtom(m_display, "_NET_CLIENT_LIST", False);
	Atom type;
	int form;
	unsigned long remain, len;
	unsigned char* prop = nullptr;

	if(XGetWindowProperty(m_display, XDefaultRootWindow(m_display), propWin, 0, 1024, False, XA_WINDOW, &type, &form, &len, &remain, &prop) == Success) {
		Window* wins = (Window*) prop;
		for(unsigned long i = 0; i < len; i++) {
			windows.push_back(Kronos::XWindow(*this, wins[i]));
		}
		XFree(prop);
	} else {
		Kronos::Logger::Log(Kronos::Logger::Level::ERROR, "Unable to get windows list from XGetWindowProperty");
	}

	return windows;
}
